package strathclyde.cs.relational;
import java.util.Hashtable;
import java.util.Stack;
import java.io.*;
import strathclyde.cs.relational.operators.*;
import strathclyde.cs.relational.structured.*;
import strathclyde.cs.relational.atomic.*;
/**TeX
*
* KParser
*
PART OF mbase
Started Sept 1997
Author Paul Cockshott
Copyright University of Strathclyde

  parses the query language as follows
  \begin{verbatim}
  <statement>	::= <decl> | <exp>
  <decl>		::= <letdecl> | <opdecl>
  <letdecl>		::= 'Let' <var>  '=' <exp>
  <exp>			::= (<exp>)|<nilad>|<monad>|<dyad>
  <nilad>		::= <primary> | <monad><nilad>|
					<set-exp>'|'<monad>
  <monad>		::= <dyad>'/'|<monad><monad>|
  <nilad><dyad>|<dyad><nilad>
  <dyad>		::= <monad><dyad>|<dyad><monad>|
  <dyad><join-sy><dyad>
  <primary>		::= <atom>|<set> | <tuple>
  <atom>		::= <var> |<number>|<string>
  <set>			::= '{' '}'| '{' <setlist> '}'
  <setlist>		::= <exp> | <exp> <setlist>
  <tuple>		::= '[' <tuplist> ']'
  <tuplist>		::= <exp> | <exp>  <tuplist>
  <operator>	::= <var> | <opsymbol> | <modifier><opsymbol>
  <modifier>	::= ':' |'.'
  <opsymbol>	::=  '+'| '-'| '' |'*' | ''| ','|'=' |'&'| ''|
					 '#'| ''| '/' | '|'| '_'| '$'| '<'

  \end{verbatim}
*/

public class KParser 
{
	private Lexanal lex;
	private Hashtable store;
	
	public KParser (InputStream I, Hashtable h)
		throws IOException
	{ lex = new Lexanal(I);store=h;}
	
	
	
	public KParser ()throws IOException
	{
		lex = new Lexanal(System.in);
		store = new Hashtable();
	}
	// declare the lexemes used
	private static final char TT_PLUS = '+';
	private static final char TT_MINUS = '-';
	private static final char TT_TIMES = 0xd7; // 
	private static final char TT_STAR = '*';
	private static final char TT_DIVIDE = 0xf7;//  
	private static final char TT_CONCAT = ',';
	private static final char TT_EQUALS = '=';
	private static final char TT_DCOMPOSE = '&';
	private static final char TT_LCOMPOSE = 0xa6;// 
	private static final char TT_RCOMPOSE = '#';
	private static final char TT_JOIN = ''; 
	private static final char TT_REDUCE = '/';
	private static final char TT_SELECT = '@';
	private static final char TT_SUBSCRIPT = '_';
	private static final char TT_LOAD = '$';
	private static final char TT_LT = '<';
	
	private static final char TT_MOD1 = ':';
	private static final char TT_EOL=(char)Lexanal.TT_EOL;
	
	private String letter(Object e)
	{
		if(e instanceof Niladic) return "n";
		if(e instanceof Moperator) return "m";
		if(e instanceof Doperator) return "d";
		return "?";
	}
	private Object operise (Object e)
	{	if(e instanceof Niladic)return e;
	return(e instanceof Universal ? new Suspended((Universal)e):e);
	}
	private Object exp()throws KSyntaxError
	{
		Object e1 = operise(Universal.undefinedValue);
		Object e2,e3;
		Stack stack = new Stack();
		String pattern="";
		stack.push(e1);
		//System.out.println("exp(");
		if (lex.have(TT_EOL)){}; 
		if(
			lex.have(')')||
			lex.have(']')||
			lex.peek(',')
			)throw new KSyntaxError("Null expression ");

		while (!lex.peek(TT_EOL)&&
			!lex.peek(')')&&
			!lex.peek('}')&&
			!lex.peek(']')&&
			!lex.peek(',')
			)
		{	//System.out.println(lex);
			if(lex.have(TT_MOD1)){
				// We now expect to have another operator whose
				// meaning may be modified relative to normal meaning
				if(lex.have(TT_PLUS)){
					// :+ is weighted add
					Object o = primary();
					Universal u;
					if(o instanceof Niladic){
						Niladic n =((Niladic)primary());
						u = n.eval();
					}
					else u = (Universal)o;
					if(u instanceof Number){
						stack.push(new Wadd((Number)u));
						pattern=pattern+'d';
					}else
						throw new KSyntaxError("Must be numeric weight to :+");
					
					
				}
			}
			else if (lex.have(TT_PLUS)){
				stack.push(Doperator.PLUS);
				pattern=pattern+'d';
			}
			else if (lex.have(TT_MINUS)){
				stack.push(Doperator.MINUS);
				pattern=pattern+'d';
			}
			else if (lex.have(TT_TIMES)){
				stack.push(Doperator.TIMES);
				pattern=pattern+'d';
			}
			else if (lex.have(TT_DIVIDE)){
				stack.push(Doperator.DIVIDE);
				pattern=pattern+'d';
			}
			else if (lex.have(TT_EQUALS)){
				stack.push(Doperator.EQ);
				pattern=pattern+'d';
			}else if (lex.have(TT_LT)){
				stack.push(Doperator.LT);
				pattern=pattern+'d';
			}else if (lex.have(TT_SUBSCRIPT)){
				stack.push(Doperator.SUBS);	
				pattern=pattern+'d';
			}else if (lex.have(TT_LOAD)){
				stack.push(Moperator.LOAD);	
				pattern=pattern+'m';
			}else if (lex.have(TT_REDUCE)){
				pattern=pattern+TT_REDUCE;
			}else if(lex.have(TT_SELECT)){
				pattern=pattern+TT_SELECT;
			}else
			{
				e1=operise(primary());
				pattern =pattern+letter(e1);
				stack.push(e1);
			}
			
			
			// now apply reduction rules
			boolean reducing=true;
			while(reducing){
				System.out.println(pattern);
				int patlen=pattern.length();
				if(pattern.endsWith("n"+(char)(TT_SELECT)+"m")){
					e1=stack.pop();e2=stack.pop();
					stack.push(new Select((Moperator)e1));
					stack.push(e2);
					pattern=pattern.substring(0,pattern.length()-3)+"mn";
				}
				if(pattern.endsWith("d"+TT_REDUCE)){
					stack.push(new Reduce((Doperator)stack.pop()));
					pattern=pattern.substring(0,pattern.length()-2)+"m";
				}
				if(pattern.endsWith("nd")){ // eg. 3 +
					pattern= pattern.substring(0,pattern.length()-2)+"m";
					e1=stack.pop();e2=stack.pop();
					stack.push(new Curried(((Niladic)e2).eval(),(Doperator)e1));
				}
				if(pattern.endsWith("mm")){ // operator composition
					pattern=pattern.substring(0,pattern.length()-1);
					e1=stack.pop();e2=stack.pop();
					stack.push(new Mcomposed((Moperator)e2,(Moperator)e1));
				}
				if(pattern.endsWith("dn")){ // eg. 3 +
					pattern= pattern.substring(0,pattern.length()-2)+"m";
					e1=stack.pop();e2=stack.pop();
					stack.push(new Curried((Doperator)e2,((Niladic)e1).eval()));
				} 
				
				if(pattern.endsWith("mn")){
				pattern = pattern.substring(0,pattern.length()-2)+"n";
					e1=stack.pop();e2=stack.pop();
					stack.push(new Nsuspended((Moperator)e2,(Niladic)e1));

				}
				if(pattern.equals("nn"))throw 
					new KSyntaxError("adjacent primaries not allowed");
				reducing = patlen>pattern.length();
			}
		}
		//System.out.println(")");
		
		return stack.pop();
}
public void statement()
{
	Object result;
	boolean stop=false;
	while(!stop){
		try{
			if(lex.have(Lexanal.TT_EOL)){
			} 
			else
				if(lex.have("input")){
				/**TeX
				The statement 
				\begin{verbatim}
				input <filename>
				\end{verbatim}
				causes a new parser to be forked
				off to read the file storing the variables
				in the same hash table used by the main parser.
				*/
				result=exp();
				if(result instanceof Niladic)
					
				{	
					try
					{ InputStream s=new FileInputStream(((Niladic)result).eval().toString());
					KParser np=new KParser(s,store)	 ;
					np.statement();
					}
					catch (FileNotFoundException e){
						System.out.println("**"+e);
				}}
				else throw new KSyntaxError("Filename expected");
				
			}	else
				if(lex.have("bye")||lex.have(Lexanal.TT_EOF)) {
				stop=true;
			} else
				if(lex.have("let")){
				lex.mustbe(Lexanal.TT_WORD);
				String var = lex.theString;
				lex.mustbe('=');
				result=exp();
				if(result instanceof Niladic){
					store.put(var,((Niladic)result).eval());
				} 
				else store.put(var,result);
			}
			else
			{
				result= exp(); 
				
				
				if(result instanceof Niladic){
					// time the following line
					double starttime;
					starttime=System.currentTimeMillis();
					result =((Niladic)result).eval();
					double stoptime=System.currentTimeMillis();
					System.out.println("-> "+result+"\t"+(stoptime-starttime)+"ms");
				}
				else
					if(result instanceof Universal ) {
					System.out.println("-> "+(Universal)result);
				}
				else System.out.println(result);
			}
			
			
		}
		catch (IOException e){System.out.println("**"+e);}
		
		
		catch (KSyntaxError e) {
			System.out.println("**"+e);
			try{	
				while(!lex.have(Lexanal.TT_EOL)){lex.nextsymbol();};
			}
			catch	(IOException f){System.out.println("**"+f);}
			
		}
	}
}


private Object primary()throws KSyntaxError
{	
	if( lex.have ('{') ) return set(new HashSet());
	if( lex.have ("Relation")){ 
		lex.mustbe('{');
		return set (new Hashrelation());
	}
	if( lex.have ('[') ) return tuple(); 
	if(lex.have('(')){
		Object e1=operise (exp());
		lex.mustbe(')');
		return e1;
	}

	return atom();
}
private Object atom() throws KSyntaxError
{	
	// handle reserved operator words
   if (lex.have("proj")) return Doperator.PROJ;

	if( lex.have( lex.TT_WORD) )
	{ 
		String var = lex.lastString;
		
	    // handle other words

		if(store.containsKey(var)){
			return store.get(var);
		}
		else throw new KSyntaxError("Undeclared "+var);
	}
	if( lex.have( lex.TT_NUMBER ) )
		return new Numeric(lex.theNumber);
	
	if( lex.have( lex.TT_QUOTE ) )
		return new Text( lex.theString);
	throw new KSyntaxError("Atomic value expected");
}
private Object set(Set s)throws KSyntaxError
{
	
	Niladic n = (Niladic)operise(s);
	while(!lex.have('}')){
		Object e=exp();
		boolean last = !lex.have(',');
		if (!(e instanceof Niladic)) 
			throw new KSyntaxError("Expression must return value");
		if( e instanceof Suspended &&((Suspended)e).Op() instanceof Identity)
		{
			s.addmember(((Suspended)e).Arg());
		}else {
			Moperator m=new NDreduce(n,new Append());
			n= new Nsuspended(m,(Niladic)e);
		}
		
	}
	return n;
}
private Tuple tuple() throws KSyntaxError
{	
	Object e=exp();
	Niladic n;
	if(e instanceof Universal){  
		Universal u[]=new Universal[1];
		if(e instanceof Niladic)e=((Niladic) e).eval();
		u[0]=(Universal)e;
		Tuple t = new Tuple(u);
		if(lex.have(',')) t=(Tuple)t.concat(tuple());
		else {
			lex.mustbe(']');
		}
		return (t);
	} else throw new KSyntaxError("expression must return value");
}
// this is the executable	
}

// classes used in building up values used in the parse tree
class MKtuple    // evaluates to make a tuple
extends Niladic
{	
	int len;
	MKtuple(int n){len=n;}
	
	public Universal eval(){return new Tuple(len);}
	
}

class Tupdate    // evaluates to update a tuple
extends Niladic
{	int i; Niladic mktup;Niladic field;
Tupdate(Niladic n, int l, Niladic v)
{ i=l; mktup =n; field = v;}
public Universal eval(){
	Tuple t=(Tuple)mktup.eval();
	t.field[i]=field.eval();
	return t;
}
}