/**TeX \footnote{Part of Mbase implemented in Java
  Author Nikolaos Kotsis
  Started Aug 1997
  Copyright (c) University of strathclyde}
*/

/**TeX This is the Superclass to which all other classes belong - 
 we assume that basic arithmetic is available on all types
The universal type is the supertype that contains all of the sub-types
derived from atoms, tuples and sets.


Its $\Sigma$ ( set of operations ) is
$\{ \leftarrow = > <  \leq \geq  + - \times \div$ max min show $ \# \}$. 
The class Universal has a single pre-given member {\tt undefinedValue},
which is returned when any operation returns an undefined
result.
 

These operations
are inherited by or overriden by equivalent operators in all sub-types of
universal. The reason for universal having so mUniversal operations
is that we want to be able to construct tuples whose elements
are drawn from the class universal. If we want to perform
arithmetic on tuples, the basic arithmetic operation must be
defined on all members of the class universal.

Beneath the universal class we have atomics, tuples and sets.
\section{ Operations on  Unversal}
\begin{tabular}{|l|l|l|}\cr
\bf Operator&\bf Type&\bf Explanation\cr
\cr
$a < b$&\dyad{ Universal} {Universal}{ {\bf B}}&Returns 1 if $a$ is less than
$b$, see section \ref{universal.lessthan}\cr

\cr$a = b$&\dyad{ Universal} {Universal}{ {\bf B}}&Returns 1 if $a$ equals
$b$\cr
$z\leftarrow a + b$&\dyad {Universal}{ Universal}{ Universal}&$z$ is the algebraic
sum of
 $a$ and $b$\cr
 $z\leftarrow a ++ b$&\dyad {Universal}{ Universal}{ Universal}&$z$ is the concatenation of
 $a$ and $b$ unlike addition, not a commutative operator\cr
$z\leftarrow a - b$&\dyad {Universal}{ Universal}{ Universal}&$z$ is the algebraic
difference of $a$ and $b$\cr
$z\leftarrow a \times b$&\dyad {Universal}{ Universal}{ Universal}&$z$ is the
algebraic product of $a$ and $b$\cr
$z\leftarrow a \div b$&\dyad {Universal}{ Universal}{ Universal}&$z$ is the quotient
 of $
a$ and $b$\cr
$z\leftarrow \neg a$&\monad{ Universal}{ {\bf B}}&Negation: $z=1 \Leftrightarrow a
=0$\cr
$z\leftarrow - a$&\monad{ Universal}{Universal}&Unary minus: $z= ^-1\times a$\cr

$z\leftarrow \card x$&\monad{ Universal}{{\bf R}}&$z$ is the absolute value of $x$\cr
\end{tabular}


*/



package strathclyde.cs.relational;
import strathclyde.cs.relational.atomic.*;
public interface  Universal{
	public static final Universal undefinedValue = new Numeric(Double.NaN);
	/**TeX The implementaion of the undefined value makes use
	of the format NaN, short for Not a Number, which is defined
	in the IEEE floating point standard as the undefined value. */
	public abstract Universal plus (Universal b);/**TeX implements $+$ */
	public abstract Universal concat (Universal b);/**TeX implements $++$ */
	public abstract Universal minus (Universal b);/**TeX implements $-$ */
	public abstract Universal times (Universal b);/**TeX implements $\times$ */
	public abstract Universal divide (Universal b);/**TeX implements $\div$ */
	public  Universal max (Universal b);
	public  Universal min (Universal b);

	public abstract boolean lessthan (Universal b);/**TeX implements $<$ 
	note that this returns a boolean not a universal, a boolean
	can be converted to a universal using the class Bool which
	implements universal.*/
}
