/**TeX
% this is the introduction file for
% the relational package
%----------------------- Auxilliary definitions
\def\dyad#1#2#3{(#1$\otimes$#2 $\rightarrow#3)$}
\def\monad#1#2{(#1$\rightarrow$#2)}
\def\triad#1#2#3#4{(#1$\otimes$#2$\otimes$#3 $\rightarrow#4)$}

\def\dif{\mskip-\medmuskip \mkern5mu \mathbin{\bf dif}\penalty900 \mkern5mu \mskip-\medmuskip}
\def\proj{\mskip-\medmuskip \mkern5mu \mathbin{\bf proj}\penalty900 \mkern5mu \mskip-\medmuskip}
\def\join{\mskip-\medmuskip \mkern5mu \mathbin{\Join}\penalty900 \mkern5mu \mskip-\medmuskip}
\def\tuple#1{\langle {#1} \rangle}
\def\card#1{\|#1}
\def\map{{\cal M}}
\def\reduce#1#2{#1 \cal R #2}

\def\sel#1#2{#1{ \cal S\ #2 }}
\def\rept{{\cal WHILE\ }}
This is the begining of an algebraic specification of a
database type system that will combine relational with probabalistic and
multi-dimensional database concepts. It is intended
that it can be used to build a new compressed database
management system as a successor to Hibase.


Why do we need an algebraic specification of the type system?
As Hoare says\footnote{
C.A.R. Hoare. Algebra and models. ACM Software Engineering Notes, 18(5):1-8, December 1993. }: 

     "The construction of a single mathematical model obeying an elegant set of algebraic laws is a significant intellectual
     achievement; so is the formulation of a set of algebraic laws characterising an interesting and useful set of models. 

     "But neither of these achievements is enough. We need to build up a large collection of models and algebras, covering a wide
     range of computational paradigms, appropriate for implementation either in hardware or in software, either of the present day or
     of some possible future." 

A failing of the original Hibase project was, to my mind, the fact
that the stage of preparing a formal specification of the system
was elided. The RA who would have had the mathematical training to
do this left to become a lecturer elsewhere and those who succeeded
him lacked the training in formal methods to do the task. A consequence
of this is that we are left with a type system that is organised
around the implementation rather than around the mathematical properties
of the system that we are modeling. I think that this is undesireable
on several counts:
\begin{enumerate}
\item the implementation shows through too much in the type system
\item this leads to an inflexibility in the implementation
\item the provision of a clean API is made harder - in fact there appears
to be no fully functional API in pascal at the end of the project
\item there is a failure to recognise the formal equivalence or
similarity of types at different places in the type hierarchy
\end{enumerate}

This last point applies particularly to the position of the vector
type in the heirarchy. I will argue in this paper that vectors
should properly belong beneath relations in the hierarchy.

What I intend to try and do in what follows is to define a hierarchy
of types, and for each type to define its {\em signature} that is to
say a set of operators on it, and a set of axioms that must be valid
for these operators.

\chapter{The type hierarchy}
\label{The notation}
%   NOTATION
\label{notation}
The implementation will be strongly typed, with an inheritance based
type hierarchy, using the Java programming environment.

Within this context, types will be implemented by Java classes, and
operators on the types by the methods that operate on the classes.

The broad division is into operators 
on the one hand and the structured  and atomic types on the other.
The structured and functional types form sub-lattices of their own.
\nl
Each type is a pair $(\Delta, \Sigma)$. $\Delta$ is a set of
sets of concrete denotations on different media: the printed page,
the video screen, tansmission form ( as sent
over transputer links), the working store of
the transputer. $\Sigma$ is its signature  or the set of functions that
take it as arguments or return it as results.
\chapter{Package strathclyde.cs.relational}
*/


