/**TeX A {\it Relation} is a set all of whose elements are tuples, all of which
are of the same arity and type.
It inherits all of its operations other than projection
and join from sets. A weighted relation similarly inherits
all of its operations from weighted sets. We will treat weighted
relations as the general form with unweighted relations being
treated as the special case of a relation for which all tuples
have equal weights.

  Relations add  to the set operations : Projection and Compression.
  
	
	  
		%========================================
		\subsection{Relational projection}
		
		  Let $r$ be a relation over tuples of type $\tuple{t_1,t_2,t_3,...,t_n} $ with $t_i$ being types,
		  then if
		  \begin{equation}\label{projection}x=r \proj\tuple{j,k,...}\end{equation} with $j,k,..$ integers in range $1..n$.
		  Then $x$ is a set of tuples
		  of type $\tuple{t_j,t_k,...}$ and
		  
			\begin{equation}\label{pmemb}\forall a=\tuple{p,q,...} \in x, \exists i \in r |i_j=p, i_k=q, ....,\ etc\end{equation}
			and
			\begin{equation}\forall z \in r, \exists s \in x | s_1=z_j, s_2=z_k, ....,\  etc \end{equation}
			where in the above, the set membership operator `$\in$' is interpreted as true if the
			probability of occurence of the tuple in the relation is non zero:$a\in B\Rightarrow \Pr Ba >0$.
			\subsection{Probabilities under projection}
			When projection occurs the probabilities of the projected tuples are the sum
			of the probabilities of the tuples that are projected onto them. Thus in 
			(\ref{projection}),(\ref{pmemb})
			\begin{equation}\Pr x a=\sum\Pr r i \forall i \in r |i_j=p, i_k=q, ....,\ etc\end{equation}
			\subsection{Cardinalities under projection}
			Since a weighted relation is taken to encode two things
			\begin{enumerate}
			\item A probability density function over a population characterised by 
			the fields of the tuples,
			\item The size of the population - the cardinality of the relation
			\end{enumerate}
			it follows that the population should be invariant under projection, in that 
			projection is equivalent to ignoring certain attributes of the population. Thus
			in (\ref{projection}), $\card x = \card r$.
			
			  Relations
			  \footnote{
			  Part of Mbase implemented in Java
			  Author Paul Cockshott
			  Started Aug 1997
			  Copyright (c) University of strathclyde} are abstract types
			  which extend sets by the addition of the projection operator.
			  A 
*/

package strathclyde.cs.relational.structured;
import strathclyde.cs.relational.*;
import java.util.Enumeration;
public abstract class Relation extends Wset	implements Subscriptable
{
	public String toString()
	{ 
		String r="";
		Enumeration i;
		
		for(i=members();i.hasMoreElements(); )
		{ 
			r=r.concat("\n");
			Universal u=(Universal)i.nextElement();
			r=r.concat(u.toString()+":"+P(u));
		}
		return r.concat("\n");
	}


public Set project(Universal u)
/**TeX
Projection of a set $r$ under tuple $t$ is the image
of $r$ under subscription by $t$. */
{	
	Set result;
	Enumeration n;
	if(u instanceof Tuple)
		result=EmptySet();// This returns an empty Relation
	else result=Set.EMPTYSET;//This returns an empty Set
	for(n=members();n.hasMoreElements();)
	{
		Subscriptable e=(Subscriptable)n.nextElement();		  
		result.addmember(e.subscript(u));		  
	}
	return result;
}

}